#include "MAX41470.h"

#include <stdio.h>
#include <string.h>

#include "mxc_delay.h"
#include "nvic_table.h"
#include "spi.h"

int MAX41470_Init() {
    int iStatus;

    mxc_spi_pins_t spiPins;
    spiPins.clock = 1;
    spiPins.miso = 1;
    spiPins.mosi = 1;
    spiPins.sdio2 = 0;
    spiPins.sdio3 = 0;
    spiPins.ss0 = 0;
    spiPins.ss1 = 0;
    spiPins.ss2 = 0;
    spiPins.vddioh = 1;

    iStatus = MXC_SPI_Init(MAX41470_SPI, 1, 0, 1, 0, 500000, spiPins);
    if (iStatus) {
        printf("MXC_SPI_Init failed with status code %d.", iStatus);
        return 1;
    }

    iStatus = MXC_SPI_SetDataSize(MAX41470_SPI, 8);
    if (iStatus) {
        printf("MXC_SPI_SetDataSize failed with status code %d.", iStatus);
        return 1;
    }

    iStatus = MXC_SPI_SetWidth(MAX41470_SPI, SPI_WIDTH_STANDARD);
    if (iStatus) {
        printf("MXC_SPI_SetWidth failed with status code %d.", iStatus);
        return 1;
    }

    mxc_gpio_cfg_t ss;
    ss.func = MXC_GPIO_FUNC_OUT;
    ss.mask = MAX41470_CS_PIN;
    ss.port = MAX41470_CS_GPIO;
    ss.vssel = MXC_GPIO_VSSEL_VDDIOH;
    ss.pad = MXC_GPIO_PAD_NONE;

    MXC_GPIO_OutSet(MAX41470_CS_GPIO, MAX41470_CS_PIN);
    MXC_GPIO_Config(&ss);
    MXC_GPIO_OutSet(MAX41470_CS_GPIO, MAX41470_CS_PIN);

    mxc_gpio_cfg_t pwdn;
    pwdn.func = MXC_GPIO_FUNC_OUT;
    pwdn.mask = MAX41470_PWDN_PIN;
    pwdn.port = MAX41470_PWDN_GPIO;
    pwdn.vssel = MXC_GPIO_VSSEL_VDDIOH;
    pwdn.pad = MXC_GPIO_PAD_NONE;

    MXC_GPIO_OutSet(MAX41470_PWDN_GPIO, MAX41470_PWDN_PIN);
    MXC_GPIO_Config(&pwdn);
    MXC_Delay(2000);
    MXC_GPIO_OutClr(MAX41470_PWDN_GPIO, MAX41470_PWDN_PIN);
    MXC_Delay(400);

    mxc_gpio_cfg_t data;
    data.func = MXC_GPIO_FUNC_IN;
    data.mask = MAX41470_DATA_PIN;
    data.port = MAX41470_DATA_GPIO;
    data.vssel = MXC_GPIO_VSSEL_VDDIOH;
    data.pad = MXC_GPIO_PAD_NONE;
    MXC_GPIO_Config(&data);

    data.mask = MAX41470_DCLK_PIN;
    data.port = MAX41470_DCLK_GPIO;
    MXC_GPIO_Config(&data);

    return 0;
}

int MAX41470_WriteReg(uint8_t addr, uint8_t* value, size_t valueSize) {
    int iResult;
    if (valueSize > 29) {
        return -1;
    }

    uint8_t buff[30];
    buff[0] = addr;
    memcpy(buff + 1, value, valueSize);

    uint8_t dummy = 0;

    mxc_spi_req_t req;
    req.rxData = NULL;
    req.rxLen = 0;
    req.spi = MAX41470_SPI;
    req.ssIdx = 0;
    req.completeCB = NULL;

    // dummy transaction whe CS=1 for prevent pulse when CS=0 next time
    req.txData = &dummy;
    req.txLen = sizeof(dummy);
    req.ssDeassert = 0;
    req.rxCnt = 0;
    req.txCnt = 0;
    MXC_SPI_MasterTransaction(&req);

    req.ssDeassert = 1;
    req.txData = buff;
    req.txLen = valueSize + 1;
    req.rxCnt = 0;
    req.txCnt = 0;

    MXC_GPIO_OutClr(MAX41470_CS_GPIO, MAX41470_CS_PIN);

    iResult = MXC_SPI_MasterTransaction(&req);

    MXC_GPIO_OutSet(MAX41470_CS_GPIO, MAX41470_CS_PIN);

    return iResult;
}